import java.util.Iterator;
import java.util.NoSuchElementException;

public class MergeIterator implements Iterator<Integer> {

  private final Iterator<Integer> it1, it2;
  private Integer x1, x2;
    // We query [it1] and [it2] in advance for their next element,
    // and store it in [x1] and [x2], respectively. We store [null]
    // if there are no more elements.
    // This approach is a bit unsatisfactory, as it queries [it1] and [it2]
    // earlier than strictly necessary, but this will do. The code would be
    // slightly more complicated if we waited until [hasNext] or [next] is
    // invoked before querying [it1] and [it2].

  public MergeIterator (Iterator<Integer> it1, Iterator<Integer> it2) {
    this.it1 = it1;
    this.it2 = it2;
    x1 = it1.hasNext() ? it1.next() : null;
    x2 = it2.hasNext() ? it2.next() : null;
  }

  public boolean hasNext() {
    return x1 != null || x2 != null;
  }

  public Integer next() {
    if (!hasNext())
      throw new NoSuchElementException();
    // At this point, [x1] and [x2] cannot both be null,
    // since [hasNext] has returned [true].

    // If [x1] is null, the next element must be [x2], which cannot be [null].
    // If [x1] is non-null and [x2] is non-null and [x2 <= x1] holds, then the
    // next element can be [x2].
    // One must be careful to perform the test [x2 <= x1] only AFTER
    // one has checked that [x1] and [x2] are both non-null. Implicit
    // conversion of the type [Integer] to the type [int] are taking
    // place here, and they can throw [NullPointerException].
    if (x1 == null || x2 != null && x2 <= x1) {
      Integer r = x2;
      x2 = it2.hasNext() ? it2.next() : null;
      return r;
    } else {
      // [x1] is non-null. [x2] could be null or non-null, but cannot
      // be non-null and smaller than [x1]. So, the next element must
      // come from [x1].
      Integer r = x1;
      x1 = it1.hasNext() ? it1.next() : null;
      return r;
    }
  }

}
